#include <stdio.h>
#include <dlfcn.h>
#include <string.h>

/*---获取设备序列号---*/
typedef int (* PFG_GetDevSn)(char * pszPort, char* extendPort, int nBaudRate, char* szDevSn);

/*---获取指纹特征---*/
typedef int (* PFG_GetFeature)(char* nPortNo, char* extendPort, int nBaudRate, int nTimeout, char* szVerBuf, int* pnVerLen, char* szErrMsg);

/*---获取指纹模板---*/
typedef int (* PFG_GetTemplate)(char* nPortNo, char* extendPort, int nBaudRate, int nTimeout, char* szRegBuf, int* pnRegLen, char* szErrMsg);

PFG_GetDevSn	FG_GetDevSn		=	NULL;
PFG_GetFeature	FG_GetFeature	=	NULL;
PFG_GetTemplate	FG_GetTemplate	=	NULL;

int main(int argc,char *argv[])
{
	int nRet = -1, i = 0;
	char *err = NULL;
	void *pHandle = NULL;
	char extendPort[8] = "";
	char szDevSn[64] = {'\0'};
	char nPortNo[12] = "ttyS0";
 	char pRegBuf[1024] = {'\0'};
	char pVerBuf[1024] = {'\0'};
	char pErrMsg[64] = {'\0'};
	int nRegLen = 0, nVerLen = 0;
	int nBaudRate = 9600, nTimeout = 30;
 	
	if(2 != argc)
	{
		printf("sudo ./main ../lib/FNG_REV1_TESO.so\n");
		return 0;
	}

	/*---加载动态库---*/
	pHandle = dlopen(argv[1], RTLD_NOW);
 

	if(NULL == pHandle)
	{
		printf("[%s:%d] 打开%s失败[%s]\n", __FILE__, __LINE__, argv[1], dlerror());	
		printf("按任意键退出...\n");
		getchar();
		return -1;
	}	

	/*---清空信息---*/
	dlerror();
	 
	/*---获取设备序列号---*/
	FG_GetDevSn = (PFG_GetDevSn)dlsym(pHandle, "FG_GetDevSn");
	err = dlerror();
	if((NULL == FG_GetDevSn) && (NULL != err))
	{
		printf("[%s:%d] 获取函数指针FG_GetDevSn指针失败[%s]\n", __FILE__, __LINE__, err);	
		nRet = -2; goto TheEnd;
	}
	
	/*---获取指纹特征---*/
	FG_GetFeature = (PFG_GetFeature)dlsym(pHandle, "FG_GetFeature");
	err = dlerror();
	if((NULL == FG_GetFeature) && (NULL != err))
	{
		printf("[%s:%d] 获取函数指针FG_GetFeature指针失败[%s]\n", __FILE__, __LINE__, err);	
		nRet = -2; goto TheEnd;
	}
 	
	/*---获取指纹模板---*/
	FG_GetTemplate = (PFG_GetTemplate)dlsym(pHandle, "FG_GetTemplate");
	err = dlerror();
	if((NULL == FG_GetTemplate) && (NULL != err))
	{
		printf("[%s:%d] 获取函数指针FG_GetTemplate指针失败[%s]\n", __FILE__, __LINE__, err);	
		nRet = -2; goto TheEnd;
	}
 
	/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

	printf("正在尝试自动打开指纹设备,请稍等...\n");
	/*---自动尝试打开设备(如果失败则手动输入)---*/
	for(i = 0;i < 4; i++)
	{
		memset(nPortNo, '\0', sizeof(nPortNo));
		memset(szDevSn, '\0', sizeof(szDevSn));
		sprintf(nPortNo,"ttyAMA%d", i);
		nRet = FG_GetDevSn(nPortNo, extendPort, nBaudRate, szDevSn);
		if(0 == nRet){break;}
	}

	if(nRet == 0)
	{
		printf("[%s:%d]指纹设备序列号:[%s]\n", __FILE__, __LINE__, szDevSn);
	}else{

		printf("[%s:%d]自动打开失败,请输出端口号(例如:ttyAMA0,ttyAMA1,ttyUSB0,...):", __FILE__, __LINE__);
		memset(nPortNo, '\0', sizeof(nPortNo));
		scanf("%s",nPortNo);
	}
 
	printf("开始获取指纹模板数据,请先将手指按捺在指纹仪上,然后按回车继续...\n");
	getchar();
	nRet = FG_GetTemplate(nPortNo, extendPort, nBaudRate, nTimeout, pRegBuf, &nRegLen, pErrMsg);
	if(0 != nRet)
	{
		printf("[%s:%d]获取指纹模板数据失败[%d]\n", __FILE__, __LINE__, nRet);
		goto TheEnd;	
	}
	printf("模板数据[%s][%d]\n",pRegBuf, nRegLen);
 

	printf("开始获取指纹特征数据...,按回车继续...\n");
	getchar();
	memset(pErrMsg, '\0', sizeof(pErrMsg));
	nRet = FG_GetFeature(nPortNo, extendPort, nBaudRate, nTimeout, pVerBuf, &nVerLen, pErrMsg);
	if(0 != nRet)
	{
		printf("[%s:%d]获取指纹特征数据失败[%d]\n", __FILE__, __LINE__, nRet);
		goto TheEnd;	
	}
	printf("特征数据[%s][%d]\n",pVerBuf, nVerLen);
 
TheEnd:

	/*---卸载动态库---*/
	if(NULL != pHandle)
	{
		dlclose(pHandle); pHandle = NULL;
	}
	printf("按任意键退出...\n");
	getchar();
	return 0;
}


